@@ -2,7 +2,7 @@ import numpy as np
 import pandas as pd
 from scipy.special import expit
 from scipy.special import logit
-from scipy.stats import pareto
+from scipy.stats import lognorm
 from itertools import product
 import sys
 from operator import mul
@@ -48,7 +48,7 @@ def normalize_adjustment(feature_values):
 
 
 
-def get_sample_times(bucket,measurment_occaisons,feature_values,sampling_param=None,sampling_function=None):
+def get_sample_times(bucket,measurment_occaisons,feature_values,abnormal_ratio,measure_max,sampling_function=None):
     if bucket == "equal":
         return(np.linspace(0,0.99999,measurment_occaisons))
     elif bucket == "random":
@@ -67,7 +67,8 @@ def get_sample_times(bucket,measurment_occaisons,feature_values,sampling_param=N
         neither=np.logical_not(either)
 
         precent_ab=np.sum(either)/measurment_occaisons
-        step_size=((0.5+0.5*precent_ab)/(measurment_occaisons-1))*np.ones(measurment_occaisons)
+
+        step_size=(((1-abnormal_ratio)+abnormal_ratio*precent_ab*(measurment_occaisons/measure_max))/(measurment_occaisons-1))*np.ones(measurment_occaisons)
 
         for feature in feature_values:
             abnormal_x[feature]=(step_size/normalize_adjustment(np.abs(feature_values[feature])))*either
@@ -128,14 +129,17 @@ def get_stationarity_change_points(stationarity_count):
     return(np.random.uniform(low=0.0, high=1.0, size=stationarity_count))
 
 
-def binary_y(arr_y):
-    draw=np.random.uniform(0,1,arr_y.size)
-    return((draw<=arr_y).astype(int))
+def binary_y(arr_y,probability_threshold):
+    if probability_threshold==None:
+        draw=np.random.uniform(0,1,arr_y.size)
+        return((draw<=arr_y).astype(int))
+    else:
+        return((probability_threshold<=arr_y).astype(int))
 
 
 class patient_model():
 
-    def __init__(self,period_index,b_values,coefficient_values,link_fn,obs_error,times,first_obs_index,relative_time,period_features,period_extraneous_variables,stationarity_trend_bucket):
+    def __init__(self,period_index,b_values,coefficient_values,link_fn,obs_error,times,first_obs_index,relative_time,period_features,period_extraneous_variables,stationarity_trend_bucket,probability_threshold):
         self.b_values={}
         self.coefficient_values={}
         for b in b_values:
@@ -153,6 +157,7 @@ class patient_model():
         self.obs_error=obs_error
         self.relative_time=relative_time
         self.stationarity_trend_bucket=stationarity_trend_bucket
+        self.probability_threshold=probability_threshold
 
     def generate_data(self):
         y=np.ones(self.num_obs)*self.coefficient_values["intercept"]
@@ -184,7 +189,7 @@ class patient_model():
             self.y=np.exp(y)+self.obs_error
         elif self.link_fn=="logit":
             self.y_prob=np.maximum(expit(y),self.obs_error)
-            self.y=binary_y(self.y_prob)
+            self.y=binary_y(self.y_prob,self.probability_threshold)
         elif self.link_fn=="inverse":
             self.y=np.power(y,-1)+self.obs_error
 
@@ -207,7 +212,7 @@ class patient_model():
 
 class patient:
 
-    def __init__(self,pat_id,b_values,features,coefficient_values,extraneous_variables,colinearity,stationarity_change_points,measurements,sampling_bucket,link_fn,sigma_e,stationarity_trend_bucket,sampling_function=None):
+    def __init__(self,pat_id,b_values,features,coefficient_values,extraneous_variables,colinearity,stationarity_change_points,measurements,sampling_bucket,link_fn,sigma_e,stationarity_trend_bucket,sampling_function,probability_threshold,abnormal_ratio,measure_max):
         self.b_values=b_values
         self.id=pat_id
         self.coefficient_values=coefficient_values
@@ -259,7 +264,7 @@ class patient:
             for variable in extraneous_variables:
                 extraneous_variable_values[variable]=x[:,index]
                 index+=1
-        time_points=get_sample_times(sampling_bucket,self.measure_count,feature_values,sampling_function)
+        time_points=get_sample_times(sampling_bucket,self.measure_count,feature_values,abnormal_ratio,measure_max,sampling_function)
         time_points=np.sort(time_points)
 
         if (sampling_bucket!="not-random") & (sampling_bucket!="custom-feature-values"):
@@ -294,7 +299,7 @@ class patient:
             for variable in extraneous_variable_values:
                 period_extraneous_variables[variable]=extraneous_variable_values[variable][key]
 
-            self.models.append(patient_model(period_index,self.b_values,self.coefficient_values,link_fn,np.array(obs_error[key]),sorted_times[key],obs_index,relative_time[key],period_features,period_extraneous_variables,stationarity_trend_bucket))
+            self.models.append(patient_model(period_index,self.b_values,self.coefficient_values,link_fn,np.array(obs_error[key]),sorted_times[key],obs_index,relative_time[key],period_features,period_extraneous_variables,stationarity_trend_bucket,probability_threshold))
             period_index+=1
             obs_index+=len(sorted_times[key])
 
@@ -317,7 +322,7 @@ class patient:
 
 class long_data_set:
 
-    def __init__(self,n=2000,num_measurements=25,collinearity_bucket="low-low",trend_bucket="linear",sampling_bucket="random",sampling_function=None,b_colin=0.13,beta_var=1,b_var=1,time_importance_factor=3,sigma_e=0.05,num_features=2,num_extraneous_variables=0,link_fn="identity",num_piecewise_breaks=0,random_effects=["intercept","time","trend-time"],coefficient_values={},time_breaks=[]):
+    def __init__(self,n=2000,num_measurements=25,collinearity_bucket="low-low",trend_bucket="linear",sampling_bucket="random",sampling_function=None,b_colin=0.13,beta_var=1,b_var=1,time_importance_factor=3,sigma_e=0.05,num_features=2,num_extraneous_variables=0,link_fn="identity",num_piecewise_breaks=0,random_effects=["intercept","time","trend-time"],coefficient_values={},time_breaks=[],probability_threshold=None):
         self.num_of_patients=n
         self.num_measurements=num_measurements
         self.colinearity_bucket=collinearity_bucket
@@ -333,6 +338,7 @@ class long_data_set:
         self.sampling_function=sampling_function
         ###############################
         self.features=[]
+        self.probability_threshold=probability_threshold
         for i in range(num_features):
             self.features.append("x"+str(i+1))
 
@@ -353,7 +359,7 @@ class long_data_set:
                 self.change_points=np.sort(self.time_breaks)
         else:
             self.change_points=np.sort(get_stationarity_change_points(self.num_piecewise_breaks))
-        measures=pareto.rvs(3.5, loc=self.num_measurements-self.num_measurements/10, scale=2.5, size=self.num_of_patients, random_state=None)
+        measures=lognorm.rvs(0.75,loc=self.num_measurements-5, scale=5, size=self.num_of_patients, random_state=None)
         measures=np.sort(np.round(measures,0))
 
         ro_x=get_colinearity(self.colinearity_bucket,self.num_of_patients)
@@ -400,11 +406,26 @@ class long_data_set:
                     else:
                         self.coefficient_values[feature][i]=np.random.normal(self.coefficient_values[feature][i-1],self.beta_var)
 
+        abnormal_ratio=0
+        for i in range(19):
+            ratio=1.0/float(i+2)
+            min_step=(1.0-ratio)/(np.min(measures)-1)
+            ab_step=0.91/(np.max(measures)-1)
+            if min_step > (ab_step):
+                if ratio > abnormal_ratio:
+                    abnormal_ratio=ratio
+            ratio=1.0-(1.0/float(i+2))
+            ab_step=0.91/(np.max(measures)-1)
+            if min_step > ab_step:
+                if ratio > abnormal_ratio:
+                    abnormal_ratio=ratio
+        self.abnormal_ratio=abnormal_ratio
+
         for p_id in range(self.num_of_patients):
             b_values={}
             for b in self.b_dict:
                 b_values[b]=self.b_dict[b][p_id]
-            pat=patient(p_id,b_values,self.features,self.coefficient_values,self.extraneous_variables,ro_x,self.change_points,measures[p_id],self.sampling_bucket,self.link_fn,self.sigma_e,self.stationarity_trend_bucket,self.sampling_function)
+            pat=patient(p_id,b_values,self.features,self.coefficient_values,self.extraneous_variables,ro_x,self.change_points,measures[p_id],self.sampling_bucket,self.link_fn,self.sigma_e,self.stationarity_trend_bucket,self.sampling_function,self.probability_threshold,abnormal_ratio,np.max(measures))
             if first:
                 first=False
                 long_data=pat.export_to_data_frame()
@@ -418,9 +439,9 @@ class long_data_set:
         self.data_frame.to_csv(path_name+"data_"+file_name+".csv")
         param_values={"piecewise_shifts":np.append(self.change_points,[1]),"cons_":self.coefficient_values["intercept"],"time_linear":self.coefficient_values["time"]}
         if self.stationarity_trend_bucket=="quadratic":
-            param_values["cos_time"]=self.coefficient_values["trend-time"]
-        elif self.stationarity_trend_bucket=="seasonal":
             param_values["sqrt_time"]=self.coefficient_values["trend-time"]
+        elif self.stationarity_trend_bucket=="seasonal":
+            param_values["cos_time"]=self.coefficient_values["trend-time"]
         for feature in self.features:
             param_values[feature]=self.coefficient_values[feature]
 
